@@ -165,7 +165,6 @@ function(object, header       = TRUE,
                  standardized = FALSE,
                  rsquare      = FALSE,
                  std.nox      = FALSE,
-                 modindices   = FALSE,
                  psrf         = TRUE,
                  neff         = FALSE,
                  postmedian   = FALSE,
@@ -209,28 +208,16 @@ function(object, header       = TRUE,
         ## 95% HPD; FIXME display blanks for equality-constrained parameters
         ##                (like Std.Err column)
 
-        ## remove rho parameters from PE
-        rhos <- grep("rho", object@ParTable$jlabel[object@ParTable$op != "=="])
-        if(length(rhos) > 0) PE <- PE[-rhos,]
-        ## move rho priors to covariance rows
-        rhos <- grep("rho", object@external$runjags$origpt$jlabel)
-        covrhos <- grep("@rho", object@external$runjags$origpt$plabel)
-        object@ParTable$prior[covrhos] <- object@external$runjags$origpt$prior[rhos]
-        ## remove equality constraints from ParTable (rhos removed in blavaan())
-        eqc <- which(object@ParTable$op == "==")
-        if(length(eqc) > 0){
-            newpt <- lapply(object@ParTable, function(x) x[-eqc])
-        } else {
-            newpt <- object@ParTable
-        }
+        ## TODO put parameter priors in partable
+
+        newpt <- object@ParTable
 
         ## match jags names to partable, then partable to PE
-        ptentry <- match(rownames(object@external$runjags$HPD), newpt$jlabel) #object@ParTable$jlabel)
-        pte2 <- ptentry[!is.na(ptentry)]
+        pte2 <- which(!is.na(newpt$jagpnum))
         peentry <- match(with(newpt, paste(lhs[pte2], op[pte2], rhs[pte2], group[pte2], sep="")),
                          paste(PE$lhs, PE$op, PE$rhs, PE$group, sep=""))
-        PE$ci.lower[peentry] <- object@external$runjags$HPD[!is.na(ptentry),'Lower95']
-        PE$ci.upper[peentry] <- object@external$runjags$HPD[!is.na(ptentry),'Upper95']
+        PE$ci.lower[peentry] <- object@external$runjags$HPD[newpt$jagpnum[pte2],'Lower95']
+        PE$ci.upper[peentry] <- object@external$runjags$HPD[newpt$jagpnum[pte2],'Upper95']
 
         ## NB This is done so that we can remove fixed parameter hpd intervals without
         ##    making changes to lavaan's print.lavaan.parameterEstimates(). But maybe
@@ -245,13 +232,13 @@ function(object, header       = TRUE,
 
         ## FIXME defined parameters never get psrf + others;
         ## see line 200 of lav_print.R
-        if(psrf & class(object@external$runjags$psrf) != "character"){
+        if(psrf){
           PE$psrf <- rep(NA, nrow(PE))
-          PE$psrf[peentry] <- object@external$runjags$psrf$psrf[!is.na(ptentry),'Point est.']
+          PE$psrf[peentry] <- newpt$psrf[pte2]
         }
         if(neff){
           PE$neff <- rep(NA, nrow(PE))
-          PE$neff[peentry] <- object@external$runjags$summaries[!is.na(ptentry),'SSeff']
+          PE$neff[peentry] <- object@external$runjags$summaries[newpt$jagpnum[pte2],'SSeff']
         }
         if(priors){
           PE$prior <- rep(NA, nrow(PE))
@@ -260,11 +247,11 @@ function(object, header       = TRUE,
         }
         if(postmedian){
           PE$Post.Med <- rep(NA, nrow(PE))
-          PE$Post.Med[peentry] <- object@external$runjags$summaries[!is.na(ptentry),'Median']
+          PE$Post.Med[peentry] <- object@external$runjags$summaries[newpt$jagpnum[pte2],'Median']
         }
         if(postmode){
           PE$Post.Mode <- rep(NA, nrow(PE))
-          PE$Post.Mode[peentry] <- object@external$runjags$summaries[!is.na(ptentry),'Mode']
+          PE$Post.Mode[peentry] <- object@external$runjags$summaries[newpt$jagpnum[pte2],'Mode']
           if(all(is.na(PE$Post.Mode))) warning("blavaan WARNING: Posterior modes require installation of the modeest package.")
         }
         if(bf){
@@ -281,8 +268,10 @@ function(object, header       = TRUE,
         }
         ## alternative names because this is not ML
         penames <- names(PE)
-        ## FIXME we need an est column for print.lavaan if we have constraints
-        names(PE)[penames == "est"] <- "Post.Mean"
+        ## This could be called "Post.Mean" except constraints
+        ## require "est"
+        #names(PE)[penames == "est"] <- "Post.Mean"
+        #PE$est <- PE$Post.Mean
         names(PE)[penames == "se"] <- "Post.SD"
         names(PE)[penames == "ci.lower"] <- "HPD.025"
         names(PE)[penames == "ci.upper"] <- "HPD.975"
@@ -290,15 +279,6 @@ function(object, header       = TRUE,
         print(PE, nd = nd)
 
     } # parameter estimates
-
-    # modification indices?
-    if(modindices) {
-        cat("Modification Indices:\n\n")
-        object@Options$estimator <- "ML"
-        object@Fit@test[[2]] <- NULL
-        print( modificationIndices(object, standardized=TRUE) )
-    }
-
 })
 
 
@@ -382,7 +362,7 @@ function(object, header       = TRUE,
 
 plot.blavaan <- function(x, pars, plot.type="trace", ...){
     # NB: arguments go to plot.runjags()
-    parnames <- rownames(x@external$runjags$summaries)[pars]
+    parnames <- x@ParTable$pxnames[match(pars, x@ParTable$free)]
     plot(x@external$runjags, plot.type=plot.type, vars=parnames, ...)
 }
     
